@@ -1,3 +1,4 @@
+/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
  * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
@@ -20,6 +21,7 @@
  *
  * Contributor(s):
  *   Roger B. Sidje <rbs@maths.uq.edu.au>
+ *   Karl Tomlinson <karlt+@karlt.net>, Mozilla Corporation
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either of the GNU General Public License Version 2 or later (the "GPL"),
@@ -244,7 +246,7 @@ SetOperator(OperatorData*   aOperatorData,
   gOperatorTable->Put(&key, aOperatorData);
 
 #ifdef NS_DEBUG
-  NS_LossyConvertUCS2toASCII str(aAttributes);
+  NS_LossyConvertUTF16toASCII str(aAttributes);
 #endif
   // Loop over the space-delimited list of attributes to get the name:value pairs
   aAttributes.Append(kNullCh);  // put an extra null at the end
@@ -297,10 +299,10 @@ InitOperators(void)
   nsCOMPtr<nsIPersistentProperties> mathfontProp;
   rv = NS_LoadPersistentPropertiesFromURISpec(getter_AddRefs(mathfontProp),
        NS_LITERAL_CSTRING("resource://gre/res/fonts/mathfont.properties"));
-  if NS_FAILED(rv) return rv;
+  if (NS_FAILED(rv)) return rv;
 
   // Get the list of invariant chars
-  for (PRInt32 i = 0; i < nsMathMLOperators::eMATHVARIANT_COUNT; ++i) {
+  for (PRInt32 i = 0; i < eMATHVARIANT_COUNT; ++i) {
     nsCAutoString key(NS_LITERAL_CSTRING("mathvariant."));
     key.Append(kMathVariant_name[i]);
     nsAutoString value;
@@ -382,27 +384,32 @@ InitGlobals()
       rv = InitOperators();
     }
   }
-  if (NS_FAILED(rv)) {
-    if (gInvariantCharArray) {
-      delete gInvariantCharArray;
-      gInvariantCharArray = nsnull;
-    }
-    if (gOperatorArray) {
-      delete[] gOperatorArray;
-      gOperatorArray = nsnull;
-    }
-    if (gStretchyOperatorArray) {
-      delete gStretchyOperatorArray;
-      gStretchyOperatorArray = nsnull;
-    }
-    if (gOperatorTable) {
-      delete gOperatorTable;
-      gOperatorTable = nsnull;
-    }
-  }
+  if (NS_FAILED(rv))
+    nsMathMLOperators::CleanUp();
   return rv;
 }
 
+void
+nsMathMLOperators::CleanUp()
+{
+  if (gInvariantCharArray) {
+    delete gInvariantCharArray;
+    gInvariantCharArray = nsnull;
+  }
+  if (gOperatorArray) {
+    delete[] gOperatorArray;
+    gOperatorArray = nsnull;
+  }
+  if (gStretchyOperatorArray) {
+    delete gStretchyOperatorArray;
+    gStretchyOperatorArray = nsnull;
+  }
+  if (gOperatorTable) {
+    delete gOperatorTable;
+    gOperatorTable = nsnull;
+  }
+}
+
 void
 nsMathMLOperators::AddRefTable(void)
 {
@@ -413,18 +420,7 @@ void
 nsMathMLOperators::ReleaseTable(void)
 {
   if (0 == --gTableRefCount) {
-    if (gOperatorArray) {
-      delete[] gOperatorArray;
-      gOperatorArray = nsnull;
-    }
-    if (gStretchyOperatorArray) {
-      delete gStretchyOperatorArray;
-      gStretchyOperatorArray = nsnull;
-    }
-    if (gOperatorTable) {
-      delete gOperatorTable;
-      gOperatorTable = nsnull;
-    }
+    CleanUp();
   }
 }
 
@@ -617,22 +613,46 @@ nsMathMLOperators::DisableStretchyOperatorAt(PRInt32 aIndex)
   }
 }
 
-PRBool
-nsMathMLOperators::LookupInvariantChar(PRUnichar     aChar,
-                                       eMATHVARIANT* aType)
+/* static */ eMATHVARIANT
+nsMathMLOperators::LookupInvariantChar(const nsAString& aChar)
 {
   if (!gInitialized) {
     InitGlobals();
   }
-  if (aType) *aType = eMATHVARIANT_NONE;
   if (gInvariantCharArray) {
     for (PRInt32 i = gInvariantCharArray->Count()-1; i >= 0; --i) {
       nsString* list = gInvariantCharArray->StringAt(i);
-      if (kNotFound != list->FindChar(aChar)) {
-        if (aType) *aType = eMATHVARIANT(i);
-        return PR_TRUE;
+      nsString::const_iterator start, end;
+      list->BeginReading(start);
+      list->EndReading(end);
+      // Style-invariant characters are at offset 3*j + 1.
+      if (FindInReadable(aChar, start, end) &&
+          start.size_backward() % 3 == 1) {
+        return eMATHVARIANT(i);
       }
     }
   }
-  return PR_FALSE;
+  return eMATHVARIANT_NONE;
+}
+
+/* static */ const nsDependentSubstring
+nsMathMLOperators::TransformVariantChar(const PRUnichar& aChar,
+                                        eMATHVARIANT aVariant)
+{
+  if (!gInitialized) {
+    InitGlobals();
+  }
+  if (gInvariantCharArray) {
+    nsString* list = gInvariantCharArray->StringAt(aVariant);
+    PRInt32 index = list->FindChar(aChar);
+    // BMP characters are at offset 3*j
+    if (index != kNotFound && index % 3 == 0 && list->Length() - index >= 2 ) {
+      // The style-invariant character is the next character
+      // (and list should contain padding if the next character is in the BMP).
+      ++index;
+      PRUint32 len = NS_IS_HIGH_SURROGATE(list->CharAt(index)) ? 2 : 1;
+      return nsDependentSubstring(*list, index, len);
+    }
+  }
+  return nsDependentSubstring(&aChar, &aChar + 1);  
 }
