@@ -38,15 +38,10 @@
 #include "nsView.h"
 #include "nsIWidget.h"
 #include "nsViewManager.h"
-#include "nsIWidget.h"
-#include "nsIButton.h"
 #include "nsGUIEvent.h"
 #include "nsIDeviceContext.h"
 #include "nsIComponentManager.h"
-#include "nsIRenderingContext.h"
-#include "nsTransform2D.h"
 #include "nsIScrollableView.h"
-#include "nsVoidArray.h"
 #include "nsGfxCIID.h"
 #include "nsIRegion.h"
 #include "nsIInterfaceRequestor.h"
@@ -62,7 +57,7 @@ static nsEventStatus PR_CALLBACK HandleEvent(nsGUIEvent *aEvent);
 
 // {34297A07-A8FD-d811-87C6-000244212BCB}
 #define VIEW_WRAPPER_IID \
-{ 0x34297a07, 0xa8fd, 0xd811, { 0x87, 0xc6, 0x0, 0x2, 0x44, 0x21, 0x2b, 0xcb } };
+{ 0x34297a07, 0xa8fd, 0xd811, { 0x87, 0xc6, 0x0, 0x2, 0x44, 0x21, 0x2b, 0xcb } }
 
 
 /**
@@ -71,7 +66,7 @@ static nsEventStatus PR_CALLBACK HandleEvent(nsGUIEvent *aEvent);
 class ViewWrapper : public nsIInterfaceRequestor
 {
   public:
-    NS_DEFINE_STATIC_IID_ACCESSOR(VIEW_WRAPPER_IID)
+    NS_DECLARE_STATIC_IID_ACCESSOR(VIEW_WRAPPER_IID)
     NS_DECL_ISUPPORTS
     NS_DECL_NSIINTERFACEREQUESTOR
 
@@ -82,6 +77,8 @@ class ViewWrapper : public nsIInterfaceRequestor
     nsView* mView;
 };
 
+NS_DEFINE_STATIC_IID_ACCESSOR(ViewWrapper, VIEW_WRAPPER_IID)
+
 NS_IMPL_ADDREF(ViewWrapper)
 NS_IMPL_RELEASE(ViewWrapper)
 #ifndef DEBUG
@@ -99,7 +96,7 @@ NS_IMETHODIMP ViewWrapper::QueryInterface(REFNSIID aIID, void** aInstancePtr)
   *aInstancePtr = nsnull;
   
   if (aIID.Equals(NS_GET_IID(nsISupports))) {
-    *aInstancePtr = NS_STATIC_CAST(nsISupports*, this);
+    *aInstancePtr = static_cast<nsISupports*>(this);
   }
   else if (aIID.Equals(NS_GET_IID(ViewWrapper))) {
     *aInstancePtr = this;
@@ -174,8 +171,6 @@ nsEventStatus PR_CALLBACK HandleEvent(nsGUIEvent *aEvent)
   return result;
 }
 
-MOZ_DECL_CTOR_COUNTER(nsView)
-
 nsView::nsView(nsViewManager* aViewManager, nsViewVisibility aVisibility)
 {
   MOZ_COUNT_CTOR(nsView);
@@ -185,11 +180,10 @@ nsView::nsView(nsViewManager* aViewManager, nsViewVisibility aVisibility)
   // a promise that the view will paint all its pixels opaquely. Views
   // should make this promise explicitly by calling
   // SetViewContentTransparency.
-  mVFlags = NS_VIEW_FLAG_TRANSPARENT;
-  mOpacity = 1.0f;
+  mVFlags = 0;
   mViewManager = aViewManager;
-  mChildRemoved = PR_FALSE;
   mDirtyRegion = nsnull;
+  mDeletionObserver = nsnull;
 }
 
 void nsView::DropMouseGrabbing() {
@@ -206,6 +200,23 @@ nsView::~nsView()
 {
   MOZ_COUNT_DTOR(nsView);
 
+  if (this == nsViewManager::GetViewFocusedBeforeSuppression()) {
+#ifdef DEBUG_FOCUS_SUPPRESSION
+    if (GetViewManager()->IsFocusSuppressed()) {
+      printf("*** 0 INFO TODO [CPEARCE] destroying view focused before suppression, while suppressed\n");
+    }
+#endif
+    nsViewManager::SetViewFocusedBeforeSuppression(nsnull);
+  }
+  if (this == nsViewManager::GetCurrentlyFocusedView()) {
+#ifdef DEBUG_FOCUS_SUPPRESSION
+    if (GetViewManager()->IsFocusSuppressed()) {
+      printf("*** 0 INFO TODO [CPEARCE] destroying view currently focused, while suppressed\n");
+    }
+#endif
+    nsViewManager::SetCurrentlyFocusedView(nsnull);
+  }
+
   while (GetFirstChild())
   {
     nsView* child = GetFirstChild();
@@ -249,12 +260,6 @@ nsView::~nsView()
     mParent->RemoveChild(this);
   }
 
-  if (mZParent)
-  {
-    mZParent->RemoveReparentedView();
-    mZParent->Destroy();
-  }
-
   // Destroy and release the widget
   if (mWindow)
   {
@@ -263,11 +268,16 @@ nsView::~nsView()
     NS_IF_RELEASE(wrapper);
 
     mWindow->SetClientData(nsnull);
-    mWindow->Destroy();
+    if (!(mVFlags & NS_VIEW_DISOWNS_WIDGET)) {
+      mWindow->Destroy();
+    }
     NS_RELEASE(mWindow);
   }
   delete mDirtyRegion;
-  delete mClipRect;
+
+  if (mDeletionObserver) {
+    mDeletionObserver->Clear();
+  }
 }
 
 nsresult nsView::QueryInterface(const nsIID& aIID, void** aInstancePtr)
@@ -304,38 +314,6 @@ void nsIView::Destroy()
   delete this;
 }
 
-NS_IMETHODIMP nsView::Paint(nsIRenderingContext& rc, const nsRect& rect,
-                              PRUint32 aPaintFlags, PRBool &aResult)
-{
-    // Just paint, assume compositor knows what it's doing.
-    if (nsnull != mClientData) {
-      nsCOMPtr<nsIViewObserver> observer;
-      if (NS_OK == mViewManager->GetViewObserver(*getter_AddRefs(observer))) {
-        observer->Paint((nsIView *)this, rc, rect);
-      }
-    }
-        return NS_OK;
-}
-
-NS_IMETHODIMP nsView::Paint(nsIRenderingContext& rc, const nsIRegion& region,
-                              PRUint32 aPaintFlags, PRBool &aResult)
-{
-  // XXX apply region to rc
-  // XXX get bounding rect from region
-  //if (nsnull != mClientData)
-  //{
-  //  nsIViewObserver *obs;
-  //
-  //  if (NS_OK == mViewManager->GetViewObserver(obs))
-  //  {
-  //    obs->Paint((nsIView *)this, rc, rect, aPaintFlags);
-  //    NS_RELEASE(obs);
-  //  }
-  //}
-  aResult = PR_FALSE;
-  return NS_ERROR_NOT_IMPLEMENTED;
-}
-
 void nsView::SetPosition(nscoord aX, nscoord aY)
 {
   mDimBounds.x += aX - mPosX;
@@ -379,6 +357,40 @@ void nsView::ResetWidgetBounds(PRBool aRecurse, PRBool aMoveOnly,
   }
 }
 
+nsRect nsView::CalcWidgetBounds(nsWindowType aType)
+{
+  nsCOMPtr<nsIDeviceContext> dx;
+  mViewManager->GetDeviceContext(*getter_AddRefs(dx));
+  NS_ASSERTION(dx, "View manager can't be created without a device context");
+  PRInt32 p2a = dx->AppUnitsPerDevPixel();
+
+  nsRect viewBounds(mDimBounds);
+
+  if (GetParent()) {
+    // put offset into screen coordinates
+    nsPoint offset;
+    nsIWidget* parentWidget = GetParent()->GetNearestWidget(&offset);
+    viewBounds += offset;
+
+    if (parentWidget && aType == eWindowType_popup &&
+        mVis == nsViewVisibility_kShow) {
+      nsRect screenRect(0,0,1,1);
+      parentWidget->WidgetToScreen(screenRect, screenRect);
+      viewBounds += nsPoint(NSIntPixelsToAppUnits(screenRect.x, p2a),
+                            NSIntPixelsToAppUnits(screenRect.y, p2a));
+    }
+  }
+
+  nsRect newBounds(viewBounds);
+  newBounds.ScaleRoundPreservingCentersInverse(p2a);
+
+  nsPoint roundedOffset(NSIntPixelsToAppUnits(newBounds.x, p2a),
+                        NSIntPixelsToAppUnits(newBounds.y, p2a));
+  mViewToWidgetOffset = viewBounds.TopLeft() - roundedOffset;
+
+  return newBounds;
+}
+
 void nsView::DoResetWidgetBounds(PRBool aMoveOnly,
                                  PRBool aInvalidateChangedSize) {
   // The geometry of a root view's widget is controlled externally,
@@ -386,46 +398,27 @@ void nsView::DoResetWidgetBounds(PRBool aMoveOnly,
   if (mViewManager->GetRootView() == this) {
     return;
   }
+  
+  nsRect curBounds;
+  mWindow->GetBounds(curBounds);
+  nsWindowType type;
+  mWindow->GetWindowType(type);
+
+  if (curBounds.IsEmpty() && mDimBounds.IsEmpty() && type == eWindowType_popup) {
+    // Don't manipulate empty popup widgets. For example there's no point
+    // moving hidden comboboxes around, or doing X server roundtrips
+    // to compute their true screen position. This could mean that WidgetToScreen
+    // operations on these widgets don't return up-to-date values, but popup
+    // positions aren't reliable anyway because of correction to be on or off-screen.
+    return;
+  }
 
   NS_PRECONDITION(mWindow, "Why was this called??");
-  nsIDeviceContext  *dx;
-  float             t2p, p2t;
-  
-  mViewManager->GetDeviceContext(dx);
-  t2p = dx->AppUnitsToDevUnits();
-  p2t = dx->DevUnitsToAppUnits();
-  NS_RELEASE(dx);
 
-  nsPoint offset(0, 0);
-  if (GetParent()) {
-    nsIWidget* parentWidget = GetParent()->GetNearestWidget(&offset);
+  nsRect newBounds = CalcWidgetBounds(type);
 
-    nsWindowType type;
-    mWindow->GetWindowType(type);
-    if (type == eWindowType_popup) {
-      // put offset into screen coordinates
-      nsRect screenRect(0,0,1,1);
-      parentWidget->WidgetToScreen(screenRect, screenRect);
-      offset += nsPoint(NSIntPixelsToTwips(screenRect.x, p2t),
-                        NSIntPixelsToTwips(screenRect.y, p2t));
-    }
-  }
-
-  nsRect newBounds(NSTwipsToIntPixels((mDimBounds.x + offset.x), t2p),
-                   NSTwipsToIntPixels((mDimBounds.y + offset.y), t2p),
-                   NSTwipsToIntPixels(mDimBounds.width, t2p),
-                   NSTwipsToIntPixels(mDimBounds.height, t2p));
-    
-  PRBool changedPos = PR_TRUE;
-  PRBool changedSize = PR_TRUE;
-  if (!(mVFlags & NS_VIEW_FLAG_HAS_POSITIONED_WIDGET)) {
-    mVFlags |= NS_VIEW_FLAG_HAS_POSITIONED_WIDGET;
-  } else {
-    nsRect curBounds;
-    mWindow->GetBounds(curBounds);
-    changedPos = curBounds.TopLeft() != newBounds.TopLeft();
-    changedSize = curBounds.Size() != newBounds.Size();
-  }
+  PRBool changedPos = curBounds.TopLeft() != newBounds.TopLeft();
+  PRBool changedSize = curBounds.Size() != newBounds.Size();
 
   if (changedPos) {
     if (changedSize && !aMoveOnly) {
@@ -576,7 +569,6 @@ void nsView::RemoveChild(nsView *child)
         break;
       }
       prevKid = kid;
-      mChildRemoved = PR_TRUE;
 	    kid = kid->GetNextSibling();
     }
     NS_ASSERTION(found, "tried to remove non child");
@@ -592,22 +584,6 @@ void nsView::RemoveChild(nsView *child)
   }
 }
 
-NS_IMETHODIMP nsView::SetOpacity(float opacity)
-{
-  mOpacity = opacity;
-  return NS_OK;
-}
-
-NS_IMETHODIMP nsView::SetContentTransparency(PRBool aTransparent)
-{
-  if (aTransparent == PR_TRUE)
-    mVFlags |= NS_VIEW_FLAG_TRANSPARENT;
-  else
-    mVFlags &= ~NS_VIEW_FLAG_TRANSPARENT;
-
-  return NS_OK;
-}
-
 // Native widgets ultimately just can't deal with the awesome power of
 // CSS2 z-index. However, we set the z-index on the widget anyway
 // because in many simple common cases the widgets do end up in the
@@ -647,12 +623,10 @@ nsresult nsIView::CreateWidget(const nsIID &aWindowIID,
                                nsNativeWidget aNative,
                                PRBool aEnableDragDrop,
                                PRBool aResetVisibility,
-                               nsContentType aContentType)
+                               nsContentType aContentType,
+                               nsIWidget* aParentWidget)
 {
-  nsIDeviceContext  *dx;
-  nsRect            trect = mDimBounds;
-
-  if (NS_UNLIKELY(!!mWindow)) {
+  if (NS_UNLIKELY(mWindow)) {
     NS_ERROR("We already have a window for this view? BAD");
     ViewWrapper* wrapper = GetWrapperFor(mWindow);
     NS_IF_RELEASE(wrapper);
@@ -660,16 +634,17 @@ nsresult nsIView::CreateWidget(const nsIID &aWindowIID,
     NS_RELEASE(mWindow);
   }
 
-  mViewManager->GetDeviceContext(dx);
-  float scale = dx->AppUnitsToDevUnits();
+  nsView* v = static_cast<nsView*>(this);
 
-  trect *= scale;
+  nsRect trect = v->CalcWidgetBounds(aWidgetInitData
+                                     ? aWidgetInitData->mWindowType
+                                     : eWindowType_child);
 
-  nsView* v = NS_STATIC_CAST(nsView*, this);
   if (NS_OK == v->LoadWidget(aWindowIID))
   {
     PRBool usewidgets;
-
+    nsCOMPtr<nsIDeviceContext> dx;
+    mViewManager->GetDeviceContext(*getter_AddRefs(dx));
     dx->SupportsNativeWidgets(usewidgets);
 
     if (PR_TRUE == usewidgets)
@@ -686,25 +661,34 @@ nsresult nsIView::CreateWidget(const nsIID &aWindowIID,
       }
       aWidgetInitData->mContentType = aContentType;
 
-      if (aNative)
+      if (aNative && aWidgetInitData->mWindowType != eWindowType_popup)
         mWindow->Create(aNative, trect, ::HandleEvent, dx, nsnull, nsnull, aWidgetInitData);
       else
       {
         if (!initDataPassedIn && GetParent() && 
           GetParent()->GetViewManager() != mViewManager)
           initData.mListenForResizes = PR_TRUE;
-
-        nsPoint offset(0, 0);
-        nsIWidget* parentWidget = GetParent() ? GetParent()->GetNearestWidget(&offset)
-          : nsnull;
-        trect += offset;
-        if (aWidgetInitData->mWindowType == eWindowType_popup) {
-          mWindow->Create(parentWidget->GetNativeData(NS_NATIVE_WIDGET), trect,
-                          ::HandleEvent, dx, nsnull, nsnull, aWidgetInitData);
-        } else {
-          mWindow->Create(parentWidget, trect,
+        if (aParentWidget) {
+          NS_ASSERTION(aWidgetInitData->mWindowType == eWindowType_popup,
+                       "popup widget type expected");
+          mWindow->Create(aParentWidget, trect,
                           ::HandleEvent, dx, nsnull, nsnull, aWidgetInitData);
         }
+        else {
+          nsIWidget* parentWidget = GetParent() ? GetParent()->GetNearestWidget(nsnull)
+                                                : nsnull;
+          if (aWidgetInitData->mWindowType == eWindowType_popup) {
+            // Without a parent, we can't make a popup.  This can happen
+            // when printing
+            if (!parentWidget)
+              return NS_ERROR_FAILURE;
+            mWindow->Create(parentWidget->GetNativeData(NS_NATIVE_WIDGET), trect,
+                            ::HandleEvent, dx, nsnull, nsnull, aWidgetInitData);
+          } else {
+            mWindow->Create(parentWidget, trect,
+                            ::HandleEvent, dx, nsnull, nsnull, aWidgetInitData);
+          }
+        }
       }
       if (aEnableDragDrop) {
         mWindow->EnableDragDrop(PR_TRUE);
@@ -712,17 +696,21 @@ nsresult nsIView::CreateWidget(const nsIID &aWindowIID,
       
       // propagate the z-index to the widget.
       UpdateNativeWidgetZIndexes(v, FindNonAutoZIndex(v));
+    } else {
+      // We should tell the widget its size even if we don't create a
+      // native widget.  (At the moment, this doesn't really matter,
+      // but we might want it to work at some point.)
+      mWindow->Resize(trect.x, trect.y, trect.width, trect.height,
+                      PR_FALSE);
     }
   }
 
   //make sure visibility state is accurate
-  
+
   if (aResetVisibility) {
     v->SetVisibility(GetVisibility());
   }
 
-  NS_RELEASE(dx);
-
   return NS_OK;
 }
 
@@ -738,33 +726,6 @@ void nsView::SetZIndex(PRBool aAuto, PRInt32 aZIndex, PRBool aTopMost)
   }
 }
 
-NS_IMETHODIMP nsView::SetWidget(nsIWidget *aWidget)
-{
-  ViewWrapper* wrapper = new ViewWrapper(this);
-  if (!wrapper)
-    return NS_ERROR_OUT_OF_MEMORY;
-  NS_ADDREF(wrapper); // Will be released in ~nsView or upon setting a new widget
-
-  // Destroy any old wrappers if there are any
-  ViewWrapper* oldWrapper = GetWrapperFor(aWidget);
-  NS_IF_RELEASE(oldWrapper);
-  NS_IF_RELEASE(mWindow);
-
-  mWindow = aWidget;
-
-  if (nsnull != mWindow)
-  {
-    NS_ADDREF(mWindow);
-    mWindow->SetClientData(wrapper);
-  }
-
-  mVFlags &= ~NS_VIEW_FLAG_HAS_POSITIONED_WIDGET;
-
-  UpdateNativeWidgetZIndexes(this, FindNonAutoZIndex(this));
-
-  return NS_OK;
-}
-
 //
 // internal window creation functions
 //
@@ -784,7 +745,6 @@ nsresult nsView::LoadWidget(const nsCID &aClassIID)
     delete wrapper;
   }
 
-  mVFlags &= ~NS_VIEW_FLAG_HAS_POSITIONED_WIDGET;
   return rv;
 }
 
@@ -800,7 +760,7 @@ void nsIView::List(FILE* out, PRInt32 aIndent) const
     float p2t;
     nsIDeviceContext *dx;
     mViewManager->GetDeviceContext(dx);
-    p2t = dx->DevUnitsToAppUnits();
+    p2t = (float) dx->AppUnitsPerDevPixel();
     NS_RELEASE(dx);
     mWindow->GetClientBounds(windowBounds);
     windowBounds *= p2t;
@@ -818,16 +778,9 @@ void nsIView::List(FILE* out, PRInt32 aIndent) const
   nsRect brect = GetBounds();
   fprintf(out, "{%d,%d,%d,%d}",
           brect.x, brect.y, brect.width, brect.height);
-  const nsView* v = NS_STATIC_CAST(const nsView*, this);
-  if (v->IsZPlaceholderView()) {
-    fprintf(out, " z-placeholder(%p)",
-            (void*)NS_STATIC_CAST(const nsZPlaceholderView*, this)->GetReparentedView());
-  }
-  if (v->GetZParent()) {
-    fprintf(out, " zparent=%p", (void*)v->GetZParent());
-  }
-  fprintf(out, " z=%d vis=%d opc=%1.3f tran=%d clientData=%p <\n",
-          mZIndex, mVis, mOpacity, IsTransparent(), mClientData);
+  const nsView* v = static_cast<const nsView*>(this);
+  fprintf(out, " z=%d vis=%d clientData=%p <\n",
+          mZIndex, mVis, mClientData);
   for (nsView* kid = mFirstChild; kid; kid = kid->GetNextSibling()) {
     NS_ASSERTION(kid->GetParent() == this, "incorrect parent");
     kid->List(out, aIndent + 1);
@@ -866,9 +819,9 @@ nsIntPoint nsIView::GetScreenPosition() const
   if (widget) {
     nsCOMPtr<nsIDeviceContext> dx;
     mViewManager->GetDeviceContext(*getter_AddRefs(dx));
-    float t2p = dx->AppUnitsToDevUnits();
-    nsIntRect ourRect(NSTwipsToIntPixels(toWidgetOffset.x, t2p),
-                      NSTwipsToIntPixels(toWidgetOffset.y, t2p),
+    PRInt32 p2a = dx->AppUnitsPerDevPixel();
+    nsIntRect ourRect(NSAppUnitsToIntPixels(toWidgetOffset.x, p2a),
+                      NSAppUnitsToIntPixels(toWidgetOffset.y, p2a),
                       0,
                       0);
     widget->WidgetToScreen(ourRect, screenRect);
@@ -881,7 +834,7 @@ nsIWidget* nsIView::GetNearestWidget(nsPoint* aOffset) const
 {
   nsPoint pt(0, 0);
   const nsView* v;
-  for (v = NS_STATIC_CAST(const nsView*, this);
+  for (v = static_cast<const nsView*>(this);
        v && !v->HasWidget(); v = v->GetParent()) {
     pt += v->GetPosition();
   }
@@ -889,7 +842,7 @@ nsIWidget* nsIView::GetNearestWidget(nsPoint* aOffset) const
     if (aOffset) {
       *aOffset = pt;
     }
-    return NS_STATIC_CAST(const nsView*, this)->GetViewManager()->GetWidget();
+    return static_cast<const nsView*>(this)->GetViewManager()->GetWidget();
   }
 
   // pt is now the offset from v's origin to this's origin
@@ -897,7 +850,8 @@ nsIWidget* nsIView::GetNearestWidget(nsPoint* aOffset) const
   // not coincide with v's origin
   if (aOffset) {
     nsRect vBounds = v->GetBounds();
-    *aOffset = pt + v->GetPosition() -  nsPoint(vBounds.x, vBounds.y);
+    *aOffset = pt + v->GetPosition() -  nsPoint(vBounds.x, vBounds.y) +
+               v->ViewToWidgetOffset();
   }
   return v->GetWidget();
 }
@@ -913,100 +867,11 @@ PRBool nsIView::ExternalIsRoot() const
   return nsIView::IsRoot();
 }
 
-/**
- * @param aStopAtView clipping from ancestors of this view will not be applied
- * @return PR_TRUE iff we found a placeholder parent edge on the way up the view tree
- * from aView to aStopAtView
- */
-static PRBool ApplyClipRect(const nsView* aView, nsRect* aRect, PRBool aFollowPlaceholders,
-                            nsIView* aStopAtView)
+void
+nsIView::SetDeletionObserver(nsWeakView* aDeletionObserver)
 {
-  // this records the offset from the origin of the current aView
-  // to the origin of the initial aView
-  nsPoint offset(0, 0);
-  PRBool lastViewIsFloating = aView->GetFloating();
-  PRBool foundPlaceholders = PR_FALSE;
-  while (NS_STATIC_CAST(const nsIView*, aView) != aStopAtView) {
-    const nsView* parentView = aView->GetParent();
-    nsPoint offsetFromParent = aView->GetPosition();
-
-    const nsView* zParent = aView->GetZParent();
-    if (zParent) {
-      foundPlaceholders = PR_TRUE;
-      if (aFollowPlaceholders) {
-        // correct offsetFromParent to account for the fact that we're
-        // switching parentView to ZParent
-        // Note that the common case is that parentView is an ancestor of
-        // ZParent.
-        const nsView* zParentChain;
-        for (zParentChain = zParent;
-             zParentChain != parentView && zParentChain;
-             zParentChain = zParentChain->GetParent()) {
-          offsetFromParent -= zParentChain->GetPosition();
-        }
-        if (!zParentChain) {
-          // uh oh. As we walked up the tree, we missed zParent. This can
-          // happen because of odd (but legal) containing block hierarchies.
-          // Just compute the required offset from scratch; this is slow,
-          // but hopefully rarely executed.
-          offsetFromParent = nsViewManager::ComputeViewOffset(aView)
-            - nsViewManager::ComputeViewOffset(zParent);
-        }
-        parentView = zParent;
-      }
-    }
-
-    if (!parentView) {
-      break;
-    }
-
-    PRBool parentIsFloating = parentView->GetFloating();
-    if (lastViewIsFloating && !parentIsFloating) {
-      break;
-    }
-
-    // now make offset be the offset from parentView's origin to the initial
-    // aView's origin
-    offset += offsetFromParent;
-    if (parentView->GetClipChildrenToBounds(aFollowPlaceholders)) {
-      // Get the parent's clip rect (which is just the dimensions in this
-      // case) into the initial aView's coordinates
-      nsRect clipRect = parentView->GetDimensions();
-      clipRect -= offset;
-      PRBool overlap = aRect->IntersectRect(clipRect, *aRect);
-      if (!overlap) {
-        break;
-      }
-    }
-    // The child-clipping rect is applied to all *content* children.
-    // So we apply it if we're in the placeholder-following pass, or
-    // if we're in the first pass and we haven't detected any
-    // placeholders yet, in which case this geometric ancestor is also
-    // a content ancestor.
-    const nsRect* r = parentView->GetClipChildrenToRect();
-    if (r && (!foundPlaceholders || aFollowPlaceholders)) {
-      // Get the parent's clip rect into the initial aView's coordinates
-      nsRect clipRect = *r;
-      clipRect -= offset;
-      PRBool overlap = aRect->IntersectRect(clipRect, *aRect);
-      if (!overlap) {
-        break;
-      }
-    }
-
-    aView = parentView;
-    lastViewIsFloating = parentIsFloating;
+  if (mDeletionObserver && aDeletionObserver) {
+    aDeletionObserver->SetPrevious(mDeletionObserver);
   }
-
-  return foundPlaceholders;
-}
-
-nsRect nsView::GetClippedRect(nsIView* aStopAtView)
-{
-  nsRect clip = GetDimensions();
-  PRBool foundPlaceholders = ApplyClipRect(this, &clip, PR_FALSE, aStopAtView);
-  if (foundPlaceholders && !clip.IsEmpty()) {
-    ApplyClipRect(this, &clip, PR_TRUE, aStopAtView);
-  }
-  return clip;
+  mDeletionObserver = aDeletionObserver;
 }
